#!perl -T
use open ':locale';
use strict;
use warnings;
# here is a comment containing ütf8 cháraçtërs, but we ignore these
print "A noite, vovo Kowalsky ve o ima cair no pe do pinguim queixoso e vovo poe acucar no cha de tamaras do jabuti feliz.\n";
