#!perl -T
use open ':locale';
use strict;
use warnings;
# here is a comment containing látïñ1 characters
print "À noite, vovô Kowalsky vê o ímã cair no pé do pingüim queixoso e vovó põe açúcar no chá de tâmaras do jabuti feliz.\n";
